<?php

class cache
{

    public function get($keyword, $get_all = false)
    {
        //lekérdezzük a keyword-höz tartozó elérési utat
        $path = $this -> get_file_path($keyword);
        //ha nem lézezik a file, null értékkel térünk vissza
        if (!file_exists($path)) {
            return null;
        }
        //ha létezik a file, kiolvassuk a tartalmát
        $object = unserialize(file_get_contents($path));
        //majd megvizsgáljuk, hogy lejárt-e már a cache
        if (!empty($object['expired_time']) && time() >= $object['expired_time']) {
            //ha lejárt eltávolítjuk a filet
            @unlink($path);
            //futtatjuk az auto_cleanert, hogy az összes lejárt file-t kitakarítsa.
            $this -> auto_cleaner();
            return null;
        }
        //a get_all true értékre állításával nem csak az értéket,
        //hanem a teljes tömböt, létrehozási idővel, cache élettartammal,
        //és lejárati idővel adjuk vissza, egyébként csak az értéket
        if ($get_all) {
            return $object;
        } else {
            return $object['value'];
        }
    }

    public function set($keyword, $content = '', $time = 0, $rewrite = 0)
    {
        //ha nem lett megadva lejárati idő, automatikusan 1 hétre állítjuk
        if ((int) $time <= 0) {
            $time = 3600 * 24 * 7;
        }
        //elkészítjük a letárolandó objektumot, a tartalommal, írási idővel
        //a lejárati idővel és az élettartammal
        $object = array(
            "value" => $content,
            "write_time" => time(),
            "expired_in" => $time,
            "expired_time" => time() + (int) $time
        );
        //szerializáljuk az objektumot
        $value = serialize($object);

        //lekérdezzük a mentendő cache file filenevét és elérési útvonalát
        $path = $this -> get_file_path($keyword);
        //alapból mindig írjuk a cachet, kivéve, ha nincs rewrite és a file létezik
        $write = 1;
        //ezt meg is vizsgáljuk, ha a file létezik és nincs rewrite, leelenőrizzük,
        //hogy a file adatai alapján, érvényes-e még a file
        if (file_exists($path) && !$rewrite) {
            //kiolvassuk a tartalmat
            $old = unserialize(file_get_contents($path));
            //visszaállítjuk az írást, mert ha a file nem járt le,
            //akkor nem kell felülírni.
            $write = 0;
            if (!empty($old['expired_time']) && time() >= $old['expired_time']) {
                //ha lejárt a cache idő, akkor felülírjuk a filet.
                $write = 1;
            }
        }
        //ha van írási engedély a filera, belepakoljuk a tartalmat
        if ($write) {
            file_put_contents($path, $value);
        }
    }

    public function clear_cache($check = 0)
    {
        //lekérdezzük a CACHE_FOLDER /cache mappa/ tartalmát pontok nélkül (.. és .),
        //és az almappákat.
        $iter = new RecursiveIteratorIterator(new RecursiveDirectoryIterator(CACHE_FOLDER, RecursiveDirectoryIterator::SKIP_DOTS), RecursiveIteratorIterator::SELF_FIRST, RecursiveIteratorIterator::CATCH_GET_CHILD);
        //végigsétálunk a kapott tömbön
        foreach ($iter as $path => $dir) {
            //ha az adott bejegyzés mappa, akkor lekérdezzük belőle a fájlokat
            //egyéb esetben kihagyjuk
            if ($dir -> isDir()) {
                //különböző rendszerek máshogy térhetnek vissza az elérési úttal
                //emiatt levágjuk a végéről a záró "/" jelet, ha van, ha nincs
                //majd mi tesszük a végére.
                $path = rtrim($path, DIRECTORY_SEPARATOR) . DIRECTORY_SEPARATOR;
                //kilistázzuk az összes .Cache kiterjesztésű filet, amit eddig létrehoztunk
                foreach (glob($path . '*.Cache') as $file) {
                    //a metódusnak be volt állítva, hogy ellenőrizze a filet,
                    //akkor minden file esetén megvizsgálja, hogy a file lejárt-e
                    //ezt a funkciót az auto_clean használja, ami csak akkor távolítja el a filet
                    //ha az már lejárt.
                    if ($check) {
                        $object = unserialize(file_get_contents($file));
                        if (!empty($object['expired_time']) && time() >= $object['expired_time']) {
                            @unlink($file);
                        }
                    } else {
                        //ha nincs check, tehát teljes törlést kérünk,
                        //a program automatikusan minden cache filet töröl,
                        //amit talál
                        @unlink($file);
                    }
                }
            }
        }
        //a PHP fájlműveletek esetén cacheli az eredményt, így hibás értéket kaphatunk, ha ezt a cachet nem ürítjük
        //például, ha törlünk egy filet, a file_exists visszatérhet igaz értékkel, ha benn maradt a cacheben
        //ezért kényszerítjük a PHP-t, hogy ürítse a cachet.
        clearstatcache();
    }

    private function get_file_path($keyword)
    {
        //legeneráljuk a filenevet
        //a fájlnévben csak az angol ABC kis és nagybetűi valamint számok lehetnek,
        //az összes többi karaktert "_" azaz "underscore" karakterré alakítjuk,
        //majd az egészet kisbetűsre.
        $filename = strtolower(trim(preg_replace("/[^a-zA-Z0-9]+/", "_", $keyword), "_"));
        //a mappa neve minden esetben a filenév első 3 betűje lesz
        //például a "cikk_valami_cim" a "/cik/cikk_valami_cim" mappába kerül
        $folder = CACHE_FOLDER . substr($filename, 0, 3) . DIRECTORY_SEPARATOR;
        //ha nem létezik a mappa, létrehozzuk
        if (!file_exists($folder) || !is_dir($folder)) {
            mkdir($folder, 0777, 1);
        }
        //visszatérünk a file teljes elérési útvonalával, hozzáfűzzük a ".Cache" kiterjesztést
        return $folder . $filename . '.Cache';
    }

    private function auto_cleaner()
    {
        //saját rendszerünket használjuk, hogy az auto_cleaner ne fusson túl gyakran
        //ehhez először lekérdezzük, hogy futott-e az elmúlt időszakban
        $autoclean = $this -> get('run_auto_cleaner');
        //ha nincs autoclean file, futtatjuk
        if (null == $autoclean) {
            //beállítjuk 1 órára az auto_cleanert, így csak minden órában tud lefutni
            $this -> set('run_auto_cleaner', 'auto_clean', 3600);
            //majd ellenőrzéssel futtatjuk a clear_cache-t.
            //Így minden file esetén megvizsgálja, hogy kell-e törölni a filet
            $this -> clear_cache(1);
        }
    }

}
